#include "idcardimage.h"

#define FONT "Bitstream Vera Sans"

IDCardImage::IDCardImage(int W, int H, int id, QString name, QString firstname, QString lastname, QString image, QString barcode)
    : QImage( QSize (W, H) , QImage::Format_ARGB32 )
{
    // Let the debugging know what we are up to ;)
    qDebug() << "IDCardImage data:";
    qDebug() << "                  size       : " << W << " x " << H;
    qDebug() << "                  id         : " << id;
    qDebug() << "                  firstname  : " << firstname;
    qDebug() << "                  firstname  : " << firstname;
    qDebug() << "                  lastname   : " << lastname;
    qDebug() << "                  image size : " << image.size();
    qDebug() << "                  bcode size : " << barcode.size();

    bool check;

    // No draw operations are done on the actual picture/image, instead
    // a QPainter is needed to do painting inside the QImage...
    QPainter imagePainter(this);

    // Logo image and ID-photo will be loaded into the QImage pic:
    QImage pic; int x; int y;

    // Use Antialiasing on all drawing operations. PLOX.
    imagePainter.setRenderHint(QPainter::Antialiasing, true);

    // Set up an outline. 1px wide border around a simulated white card.
    imagePainter.setPen(QPen(Qt::white, 1, Qt::SolidLine, Qt::RoundCap, Qt::MiterJoin));
    imagePainter.setBrush(QBrush(Qt::white, Qt::SolidPattern));
    imagePainter.drawRect(0, 0, W ,H);

    // The red area on the top of the ID card is actually quite complex.
    // Hopefully we can drop this "brute-force work" to create the rounded heading
    // on the actual card, and simply print a square.
    imagePainter.setPen(QPen(QColor(152,2,3), 1, Qt::SolidLine, Qt::RoundCap, Qt::MiterJoin));
    imagePainter.setBrush(QBrush(QColor(152,2,3), Qt::SolidPattern));
    imagePainter.drawRect(0, 0 ,W , H / 5 );

    // Header text in white.
    imagePainter.setPen(QPen(Qt::white, 2, Qt::SolidLine, Qt::RoundCap, Qt::MiterJoin));
    imagePainter.setFont(QFont( FONT , (5*H/100), QFont::Bold, 0));
    imagePainter.drawText( W / 20 , 3 * H / 20 , "Ansattbevis for Studentenes Hus");

    // Logo image
    // TODO: This image needs to be loaded as a packaged resource, as there
    // is no clean way of telling where this image is located on a filesys.
    check = pic.load("huset.png");
    qDebug() << "Loaded logo from file: " << check;
    pic = pic.scaledToHeight( H / 5 , Qt::SmoothTransformation);

    x = pic.size().width(); y = pic.size().height();
    qDebug() << "Logo dimensions: " << x << " x " << y;
    imagePainter.drawImage( W - x - ( W/20 ) , ( H/20 ) ,pic);

    // Reset the drawing pen to black color, for writing text and boxes on the card
    imagePainter.setPen(QPen(Qt::black, 2, Qt::SolidLine, Qt::RoundCap, Qt::MiterJoin));
    imagePainter.setBrush(QBrush(Qt::white, Qt::SolidPattern));

    // Add the static labels to the card
    imagePainter.setFont(QFont( FONT , (4*H/100), QFont::Normal, 0));
    imagePainter.drawText(W / 20, 4 * H / 10 , "Fornavn:");
    imagePainter.drawText(W / 20, 6 * H / 10 , "Etternavn:");

    // Add the actual dynamic text.
    imagePainter.setFont(QFont( FONT , (5*H/100), QFont::Bold, 0));
    imagePainter.drawText(W / 10 , 5 * H / 10 , firstname);
    imagePainter.drawText( W / 10 , 7 * H / 10 , lastname);


    // Load the image data from the BASE64 endoced data in the QString "image"..
    // The image is stored as a BASE64 encoded JPEG in the database. When the
    // preview is created, the imagedata is passed along as a CTor parameter,
    // in its raw, encoded, format. Here, the very convenient loadFromData is
    // used to load up the image, and then it gets scaled down to 187x250px.
    check = pic.loadFromData( QByteArray::fromBase64(image.toAscii()) );
    qDebug() << "Loaded image from base64: " << check;
    pic = pic.scaledToHeight(H/3*2, Qt::SmoothTransformation);
    x = pic.size().width(); y = pic.size().height();
    qDebug() << "Photo dimensions: " << x << " x " << y;

    // Add a box around the ID photo
    int rightoffset = H/20;
    int topoffset = (H/5) + ( 2 * H / 20);
    imagePainter.drawRect(W - x - rightoffset - 2 , topoffset - 2 ,x+4 ,y+4);

    imagePainter.drawImage(W - x - rightoffset , topoffset ,pic);


    // Example implementation on adding a barcode to the card...
    // My current idea for the barcode generation, is to add the actual barcode
    // generation to the "send to printing" process of the web-interface, so that
    // the barcode is already generated and stored in the database when
    // we come to this application.
    // The spec of the barcode in the 'barcode' string is: Code39, 6 characters long,
    // As the barcode is generated as a neatly trimmed, base64 encoded PNG file,
    // the job of adding it to the card is the same as for the ID photo.
    check = pic.loadFromData( QByteArray::fromBase64( barcode.toAscii() ) );
    qDebug() << "Loaded barcode from base64: " << check;

    pic = pic.scaledToHeight(H/5, Qt::SmoothTransformation);
    x = pic.size().width(); y = pic.size().height();

    int bcodexoff = 3*H/20;
    int bcodeyoff = ( 7 * H / 10 ) + ( H / 20);

    // Draw a box for the barcode.
    // I wonder where I can get a good barcode implementation....
    imagePainter.drawRect(bcodexoff, bcodeyoff, x + 20, y + 4);

    qDebug() << "Barcode dimensions: " << x << " x " << y;
    imagePainter.drawImage(bcodexoff + 10, bcodeyoff + 2 ,pic);

}
